@@ -29,9 +29,9 @@ import org.apache.commons.compress.utils.IOUtils;
 
 /**
  * Implements the "ar" archive format as an input stream.
- *
+ * 
  * @NotThreadSafe
- *
+ * 
  */
 public class ArArchiveInputStream extends ArchiveInputStream {
 
@@ -54,27 +54,16 @@ public class ArArchiveInputStream extends ArchiveInputStream {
      */
     private long entryOffset = -1;
 
-    // offsets and length of meta data parts
-    private static final int NAME_OFFSET = 0;
-    private static final int NAME_LEN = 16;
-    private static final int LAST_MODIFIED_OFFSET = NAME_LEN;
-    private static final int LAST_MODIFIED_LEN = 12;
-    private static final int USER_ID_OFFSET = LAST_MODIFIED_OFFSET + LAST_MODIFIED_LEN;
-    private static final int USER_ID_LEN = 6;
-    private static final int GROUP_ID_OFFSET = USER_ID_OFFSET + USER_ID_LEN;
-    private static final int GROUP_ID_LEN = 6;
-    private static final int FILE_MODE_OFFSET = GROUP_ID_OFFSET + GROUP_ID_LEN;
-    private static final int FILE_MODE_LEN = 8;
-    private static final int LENGTH_OFFSET = FILE_MODE_OFFSET + FILE_MODE_LEN;
-    private static final int LENGTH_LEN = 10;
-
-    // cached buffer for meta data - must only be used locally in the class (COMPRESS-172 - reduce garbage collection)
-    private final byte[] metaData =
-        new byte[NAME_LEN + LAST_MODIFIED_LEN + USER_ID_LEN + GROUP_ID_LEN + FILE_MODE_LEN + LENGTH_LEN];
+    // cached buffers - must only be used locally in the class (COMPRESS-172 - reduce garbage collection)
+    private final byte[] NAME_BUF = new byte[16];
+    private final byte[] LAST_MODIFIED_BUF = new byte[12];
+    private final byte[] ID_BUF = new byte[6];
+    private final byte[] FILE_MODE_BUF = new byte[8];
+    private final byte[] LENGTH_BUF = new byte[10];
 
     /**
      * Constructs an Ar input stream with the referenced stream
-     *
+     * 
      * @param pInput
      *            the ar input stream
      */
@@ -85,7 +74,7 @@ public class ArArchiveInputStream extends ArchiveInputStream {
 
     /**
      * Returns the next AR entry in this stream.
-     *
+     * 
      * @return the next AR entry.
      * @throws IOException
      *             if the entry could not be read
@@ -93,16 +82,14 @@ public class ArArchiveInputStream extends ArchiveInputStream {
     public ArArchiveEntry getNextArEntry() throws IOException {
         if (currentEntry != null) {
             final long entryEnd = entryOffset + currentEntry.getLength();
-            long skipped = IOUtils.skip(input, entryEnd - offset);
-            trackReadBytes(skipped);
+            IOUtils.skip(this, entryEnd - offset);
             currentEntry = null;
         }
 
         if (offset == 0) {
             final byte[] expected = ArchiveUtils.toAsciiBytes(ArArchiveEntry.HEADER);
             final byte[] realized = new byte[expected.length];
-            final int read = IOUtils.readFully(input, realized);
-            trackReadBytes(read);
+            final int read = IOUtils.readFully(this, realized);
             if (read != expected.length) {
                 throw new IOException("failed to read header. Occured at byte: " + getBytesRead());
             }
@@ -113,31 +100,27 @@ public class ArArchiveInputStream extends ArchiveInputStream {
             }
         }
 
-        if (offset % 2 != 0) {
-            if (input.read() < 0) {
-                // hit eof
-                return null;
-            }
-            trackReadBytes(1);
+        if (offset % 2 != 0 && read() < 0) {
+            // hit eof
+            return null;
         }
 
         if (input.available() == 0) {
             return null;
         }
 
-        {
-            final int read = IOUtils.readFully(input, metaData);
-            trackReadBytes(read);
-            if (read < metaData.length) {
-                throw new IOException("truncated ar archive");
-            }
-        }
+        IOUtils.readFully(this, NAME_BUF);
+        IOUtils.readFully(this, LAST_MODIFIED_BUF);
+        IOUtils.readFully(this, ID_BUF);
+        int userId = asInt(ID_BUF, true);
+        IOUtils.readFully(this, ID_BUF);
+        IOUtils.readFully(this, FILE_MODE_BUF);
+        IOUtils.readFully(this, LENGTH_BUF);
 
         {
             final byte[] expected = ArchiveUtils.toAsciiBytes(ArArchiveEntry.TRAILER);
             final byte[] realized = new byte[expected.length];
-            final int read = IOUtils.readFully(input, realized);
-            trackReadBytes(read);
+            final int read = IOUtils.readFully(this, realized);
             if (read != expected.length) {
                 throw new IOException("failed to read entry trailer. Occured at byte: " + getBytesRead());
             }
@@ -153,76 +136,74 @@ public class ArArchiveInputStream extends ArchiveInputStream {
 //        GNU ar uses a '/' to mark the end of the filename; this allows for the use of spaces without the use of an extended filename.
 
         // entry name is stored as ASCII string
-        String temp = ArchiveUtils.toAsciiString(metaData, NAME_OFFSET, NAME_LEN).trim();
+        String temp = ArchiveUtils.toAsciiString(NAME_BUF).trim();
         if (isGNUStringTable(temp)) { // GNU extended filenames entry
-            currentEntry = readGNUStringTable(metaData, LENGTH_OFFSET, LENGTH_LEN);
+            currentEntry = readGNUStringTable(LENGTH_BUF);
             return getNextArEntry();
         }
 
-        long len = asLong(metaData, LENGTH_OFFSET, LENGTH_LEN);
+        long len = asLong(LENGTH_BUF);
         if (temp.endsWith("/")) { // GNU terminator
             temp = temp.substring(0, temp.length() - 1);
         } else if (isGNULongName(temp)) {
-            final int off = Integer.parseInt(temp.substring(1));// get the offset
+            int off = Integer.parseInt(temp.substring(1));// get the offset
             temp = getExtendedName(off); // convert to the long name
         } else if (isBSDLongName(temp)) {
             temp = getBSDLongName(temp);
             // entry length contained the length of the file name in
             // addition to the real length of the entry.
             // assume file name was ASCII, there is no "standard" otherwise
-            final int nameLen = temp.length();
+            int nameLen = temp.length();
             len -= nameLen;
             entryOffset += nameLen;
         }
 
-        currentEntry = new ArArchiveEntry(temp, len,
-                                          asInt(metaData, USER_ID_OFFSET, USER_ID_LEN, true),
-                                          asInt(metaData, GROUP_ID_OFFSET, GROUP_ID_LEN, true),
-                                          asInt(metaData, FILE_MODE_OFFSET, FILE_MODE_LEN, 8),
-                                          asLong(metaData, LAST_MODIFIED_OFFSET, LAST_MODIFIED_LEN));
+        currentEntry = new ArArchiveEntry(temp, len, userId,
+                                          asInt(ID_BUF, true),
+                                          asInt(FILE_MODE_BUF, 8),
+                                          asLong(LAST_MODIFIED_BUF));
         return currentEntry;
     }
 
     /**
      * Get an extended name from the GNU extended name buffer.
-     *
+     * 
      * @param offset pointer to entry within the buffer
      * @return the extended file name; without trailing "/" if present.
      * @throws IOException if name not found or buffer not set up
      */
-    private String getExtendedName(final int offset) throws IOException {
+    private String getExtendedName(int offset) throws IOException{
         if (namebuffer == null) {
             throw new IOException("Cannot process GNU long filename as no // record was found");
         }
-        for (int i = offset; i < namebuffer.length; i++) {
-            if (namebuffer[i] == '\012' || namebuffer[i] == 0) {
-                if (namebuffer[i - 1] == '/') {
+        for(int i=offset; i < namebuffer.length; i++){
+            if (namebuffer[i]=='\012'){
+                if (namebuffer[i-1]=='/') {
                     i--; // drop trailing /
                 }
-                return ArchiveUtils.toAsciiString(namebuffer, offset, i - offset);
+                return ArchiveUtils.toAsciiString(namebuffer, offset, i-offset);
             }
         }
-        throw new IOException("Failed to read entry: " + offset);
+        throw new IOException("Failed to read entry: "+offset);
+    }
+    private long asLong(byte[] input) {
+        return Long.parseLong(ArchiveUtils.toAsciiString(input).trim());
     }
 
-    private long asLong(final byte[] byteArray, int offset, int len) {
-        return Long.parseLong(ArchiveUtils.toAsciiString(byteArray, offset, len).trim());
+    private int asInt(byte[] input) {
+        return asInt(input, 10, false);
     }
 
-    private int asInt(final byte[] byteArray, int offset, int len) {
-        return asInt(byteArray, offset, len, 10, false);
+    private int asInt(byte[] input, boolean treatBlankAsZero) {
+        return asInt(input, 10, treatBlankAsZero);
     }
 
-    private int asInt(final byte[] byteArray, int offset, int len, final boolean treatBlankAsZero) {
-        return asInt(byteArray, offset, len, 10, treatBlankAsZero);
+    private int asInt(byte[] input, int base) {
+        return asInt(input, base, false);
     }
 
-    private int asInt(final byte[] byteArray, int offset, int len, final int base) {
-        return asInt(byteArray, offset, len, base, false);
-    }
-
-    private int asInt(final byte[] byteArray, int offset, int len, final int base, final boolean treatBlankAsZero) {
-        final String string = ArchiveUtils.toAsciiString(byteArray, offset, len).trim();
+    private int asInt(byte[] input, int base, boolean treatBlankAsZero) {
+        String string = ArchiveUtils.toAsciiString(input).trim();
         if (string.length() == 0 && treatBlankAsZero) {
             return 0;
         }
@@ -231,7 +212,7 @@ public class ArArchiveInputStream extends ArchiveInputStream {
 
     /*
      * (non-Javadoc)
-     *
+     * 
      * @see
      * org.apache.commons.compress.archivers.ArchiveInputStream#getNextEntry()
      */
@@ -242,7 +223,7 @@ public class ArArchiveInputStream extends ArchiveInputStream {
 
     /*
      * (non-Javadoc)
-     *
+     * 
      * @see java.io.InputStream#close()
      */
     @Override
@@ -256,44 +237,68 @@ public class ArArchiveInputStream extends ArchiveInputStream {
 
     /*
      * (non-Javadoc)
-     *
+     * 
      * @see java.io.InputStream#read(byte[], int, int)
      */
     @Override
-    public int read(final byte[] b, final int off, final int len) throws IOException {
-        if (currentEntry == null) {
-            throw new IllegalStateException("No current ar entry");
-        }
+    public int read(byte[] b, final int off, final int len) throws IOException {
         int toRead = len;
-        final long entryEnd = entryOffset + currentEntry.getLength();
-        if (len > 0 && entryEnd > offset) {
-            toRead = (int) Math.min(len, entryEnd - offset);
-        } else {
-            return -1;
+        if (currentEntry != null) {
+            final long entryEnd = entryOffset + currentEntry.getLength();
+            if (len > 0 && entryEnd > offset) {
+                toRead = (int) Math.min(len, entryEnd - offset);
+            } else {
+                return -1;
+            }
         }
         final int ret = this.input.read(b, off, toRead);
-        trackReadBytes(ret);
+        count(ret);
+        offset += ret > 0 ? ret : 0;
         return ret;
     }
 
     /**
      * Checks if the signature matches ASCII "!&lt;arch&gt;" followed by a single LF
      * control character
-     *
+     * 
      * @param signature
      *            the bytes to check
      * @param length
      *            the number of bytes to check
      * @return true, if this stream is an Ar archive stream, false otherwise
      */
-    public static boolean matches(final byte[] signature, final int length) {
+    public static boolean matches(byte[] signature, int length) {
         // 3c21 7261 6863 0a3e
 
-        return length >= 8 && signature[0] == 0x21 &&
-                signature[1] == 0x3c && signature[2] == 0x61 &&
-                signature[3] == 0x72 && signature[4] == 0x63 &&
-                signature[5] == 0x68 && signature[6] == 0x3e &&
-                signature[7] == 0x0a;
+        if (length < 8) {
+            return false;
+        }
+        if (signature[0] != 0x21) {
+            return false;
+        }
+        if (signature[1] != 0x3c) {
+            return false;
+        }
+        if (signature[2] != 0x61) {
+            return false;
+        }
+        if (signature[3] != 0x72) {
+            return false;
+        }
+        if (signature[4] != 0x63) {
+            return false;
+        }
+        if (signature[5] != 0x68) {
+            return false;
+        }
+        if (signature[6] != 0x3e) {
+            return false;
+        }
+        if (signature[7] != 0x0a) {
+            return false;
+        }
+
+        return true;
     }
 
     static final String BSD_LONGNAME_PREFIX = "#1/";
@@ -324,7 +329,7 @@ public class ArArchiveInputStream extends ArchiveInputStream {
      *
      * @since 1.3
      */
-    private static boolean isBSDLongName(final String name) {
+    private static boolean isBSDLongName(String name) {
         return name != null && name.matches(BSD_LONGNAME_PATTERN);
     }
 
@@ -336,12 +341,12 @@ public class ArArchiveInputStream extends ArchiveInputStream {
      *
      * @since 1.3
      */
-    private String getBSDLongName(final String bsdLongName) throws IOException {
-        final int nameLen =
+    private String getBSDLongName(String bsdLongName) throws IOException {
+        int nameLen =
             Integer.parseInt(bsdLongName.substring(BSD_LONGNAME_PREFIX_LEN));
-        final byte[] name = new byte[nameLen];
-        final int read = IOUtils.readFully(input, name);
-        trackReadBytes(read);
+        byte[] name = new byte[nameLen];
+        int read = IOUtils.readFully(input, name);
+        count(read);
         if (read != nameLen) {
             throw new EOFException();
         }
@@ -361,33 +366,25 @@ public class ArArchiveInputStream extends ArchiveInputStream {
      * <p>A header references an extended filename by storing a "/"
      * followed by a decimal offset to the start of the filename in
      * the extended filename data section.</p>
-     *
+     * 
      * <p>The format of the "//" file itself is simply a list of the
      * long filenames, each separated by one or more LF
      * characters. Note that the decimal offsets are number of
      * characters, not line or string number within the "//" file.</p>
      */
-    private static boolean isGNUStringTable(final String name) {
+    private static boolean isGNUStringTable(String name) {
         return GNU_STRING_TABLE_NAME.equals(name);
     }
 
-    private void trackReadBytes(final long read) {
-        count(read);
-        if (read > 0) {
-            offset += read;
-        }
-    }
-
     /**
      * Reads the GNU archive String Table.
      *
      * @see #isGNUStringTable
      */
-    private ArArchiveEntry readGNUStringTable(final byte[] length, final int offset, final int len) throws IOException {
-        final int bufflen = asInt(length, offset, len); // Assume length will fit in an int
+    private ArArchiveEntry readGNUStringTable(byte[] length) throws IOException {
+        int bufflen = asInt(length); // Assume length will fit in an int
         namebuffer = new byte[bufflen];
-        final int read = IOUtils.readFully(input, namebuffer, 0, bufflen);
-        trackReadBytes(read);
+        int read = read(namebuffer, 0, bufflen);
         if (read != bufflen){
             throw new IOException("Failed to read complete // record: expected="
                                   + bufflen + " read=" + read);
@@ -403,7 +400,7 @@ public class ArArchiveInputStream extends ArchiveInputStream {
      *
      * @see #isGNUStringTable
      */
-    private boolean isGNULongName(final String name) {
+    private boolean isGNULongName(String name) {
         return name != null && name.matches(GNU_LONGNAME_PATTERN);
     }
 }
