@@ -63,11 +63,17 @@ class TapeInputStream extends FilterInputStream {
      *             more than one block has been read
      * @throws IOException
      *             there was an error reading additional blocks.
+     * @throws IOException
+     *             recsPerBlock is smaller than 1
      */
     public void resetBlockSize(final int recsPerBlock, final boolean isCompressed)
         throws IOException {
         this.isCompressed = isCompressed;
 
+        if (recsPerBlock < 1) {
+            throw new IOException("Block with " + recsPerBlock
+                + " records found, must be at least 1");
+        }
         blockSize = RECORD_SIZE * recsPerBlock;
 
         // save first block in case we need it again
@@ -100,7 +106,7 @@ class TapeInputStream extends FilterInputStream {
     @Override
     public int read() throws IOException {
         throw new IllegalArgumentException(
-            "all reads must be multiple of record size (" + RECORD_SIZE +
+            "All reads must be multiple of record size (" + RECORD_SIZE +
             " bytes.");
     }
 
@@ -116,7 +122,7 @@ class TapeInputStream extends FilterInputStream {
     public int read(final byte[] b, int off, final int len) throws IOException {
         if ((len % RECORD_SIZE) != 0) {
             throw new IllegalArgumentException(
-                "all reads must be multiple of record size (" + RECORD_SIZE +
+                "All reads must be multiple of record size (" + RECORD_SIZE +
                 " bytes.");
         }
 
@@ -166,7 +172,7 @@ class TapeInputStream extends FilterInputStream {
     public long skip(final long len) throws IOException {
         if ((len % RECORD_SIZE) != 0) {
             throw new IllegalArgumentException(
-                "all reads must be multiple of record size (" + RECORD_SIZE +
+                "All reads must be multiple of record size (" + RECORD_SIZE +
                 " bytes.");
         }
 
@@ -266,7 +272,7 @@ class TapeInputStream extends FilterInputStream {
      */
     private void readBlock(final boolean decompress) throws IOException {
         if (in == null) {
-            throw new IOException("input buffer is closed");
+            throw new IOException("Input buffer is closed");
         }
 
         if (!isCompressed || (currBlkIdx == -1)) {
@@ -309,7 +315,7 @@ class TapeInputStream extends FilterInputStream {
                                 throw new ShortFileException();
                             }
                         } catch (final DataFormatException e) {
-                            throw new DumpArchiveException("bad data", e);
+                            throw new DumpArchiveException("Bad data", e);
                         } finally {
                             inflator.end();
                         }
