@@ -22,11 +22,15 @@ import java.io.ByteArrayInputStream;
 import java.io.Closeable;
 import java.io.DataInputStream;
 import java.io.File;
+import java.io.FilterInputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.nio.ByteBuffer;
 import java.nio.ByteOrder;
+import java.nio.CharBuffer;
 import java.nio.channels.SeekableByteChannel;
+import java.nio.charset.StandardCharsets;
+import java.nio.charset.CharsetEncoder;
 import java.nio.file.Files;
 import java.nio.file.StandardOpenOption;
 import java.util.ArrayList;
@@ -40,6 +44,7 @@ import org.apache.commons.compress.utils.BoundedInputStream;
 import org.apache.commons.compress.utils.CRC32VerifyingInputStream;
 import org.apache.commons.compress.utils.CharsetNames;
 import org.apache.commons.compress.utils.IOUtils;
+import org.apache.commons.compress.utils.InputStreamStatistics;
 
 /**
  * Reads a 7z file, using SeekableByteChannel under
@@ -66,13 +71,20 @@ import org.apache.commons.compress.utils.IOUtils;
  * encrypted, neither file names nor file
  * contents can be read, but the use of
  * encryption isn't plausibly deniable.
- * 
+ *
+ * <p>Multi volume archives can be read by concatenating the parts in
+ * correct order - either manually or by using {link
+ * org.apache.commons.compress.utils.MultiReadOnlySeekableByteChannel}
+ * for example.</p>
+ *
  * @NotThreadSafe
  * @since 1.6
  */
 public class SevenZFile implements Closeable {
     static final int SIGNATURE_HEADER_SIZE = 32;
 
+    private static final String DEFAULT_FILE_NAME = "unknown archive";
+
     private final String fileName;
     private SeekableByteChannel channel;
     private final Archive archive;
@@ -80,6 +92,10 @@ public class SevenZFile implements Closeable {
     private int currentFolderIndex = -1;
     private InputStream currentFolderInputStream = null;
     private byte[] password;
+    private final SevenZFileOptions options;
+
+    private long compressedBytesReadFromCurrentEntry;
+    private long uncompressedBytesReadFromCurrentEntry;
 
     private final ArrayList<InputStream> deferredBlockStreams = new ArrayList<>();
 
@@ -91,15 +107,42 @@ public class SevenZFile implements Closeable {
     /**
      * Reads a file as 7z archive
      *
-     * @param filename the file to read
+     * @param fileName the file to read
+     * @param password optional password if the archive is encrypted
+     * @throws IOException if reading the archive fails
+     * @since 1.17
+     */
+    public SevenZFile(final File fileName, final char[] password) throws IOException {
+        this(fileName, password, SevenZFileOptions.DEFAULT);
+    }
+
+    /**
+     * Reads a file as 7z archive with additional options.
+     *
+     * @param fileName the file to read
+     * @param password optional password if the archive is encrypted
+     * @param options the options to apply
+     * @throws IOException if reading the archive fails or the memory limit (if set) is too small
+     * @since 1.19
+     */
+    public SevenZFile(final File fileName, final char[] password, SevenZFileOptions options) throws IOException {
+        this(Files.newByteChannel(fileName.toPath(), EnumSet.of(StandardOpenOption.READ)),
+                fileName.getAbsolutePath(), utf16Decode(password), true, options);
+    }
+
+    /**
+     * Reads a file as 7z archive
+     *
+     * @param fileName the file to read
      * @param password optional password if the archive is encrypted -
      * the byte array is supposed to be the UTF16-LE encoded
      * representation of the password.
      * @throws IOException if reading the archive fails
+     * @deprecated use the char[]-arg version for the password instead
      */
-    public SevenZFile(final File filename, final byte[] password) throws IOException {
-        this(Files.newByteChannel(filename.toPath(), EnumSet.of(StandardOpenOption.READ)),
-             filename.getAbsolutePath(), password, true);
+    public SevenZFile(final File fileName, final byte[] password) throws IOException {
+        this(Files.newByteChannel(fileName.toPath(), EnumSet.of(StandardOpenOption.READ)),
+                fileName.getAbsolutePath(), password, true, SevenZFileOptions.DEFAULT);
     }
 
     /**
@@ -114,7 +157,130 @@ public class SevenZFile implements Closeable {
      * @since 1.13
      */
     public SevenZFile(final SeekableByteChannel channel) throws IOException {
-        this(channel, "unknown archive", null);
+        this(channel, SevenZFileOptions.DEFAULT);
+    }
+
+    /**
+     * Reads a SeekableByteChannel as 7z archive with addtional options.
+     *
+     * <p>{@link
+     * org.apache.commons.compress.utils.SeekableInMemoryByteChannel}
+     * allows you to read from an in-memory archive.</p>
+     *
+     * @param channel the channel to read
+     * @param options the options to apply
+     * @throws IOException if reading the archive fails or the memory limit (if set) is too small
+     * @since 1.19
+     */
+    public SevenZFile(final SeekableByteChannel channel, SevenZFileOptions options) throws IOException {
+        this(channel, DEFAULT_FILE_NAME, (char[]) null, options);
+    }
+
+    /**
+     * Reads a SeekableByteChannel as 7z archive
+     *
+     * <p>{@link
+     * org.apache.commons.compress.utils.SeekableInMemoryByteChannel}
+     * allows you to read from an in-memory archive.</p>
+     *
+     * @param channel the channel to read
+     * @param password optional password if the archive is encrypted
+     * @throws IOException if reading the archive fails
+     * @since 1.17
+     */
+    public SevenZFile(final SeekableByteChannel channel,
+                      final char[] password) throws IOException {
+        this(channel, password, SevenZFileOptions.DEFAULT);
+    }
+
+    /**
+     * Reads a SeekableByteChannel as 7z archive with additional options.
+     *
+     * <p>{@link
+     * org.apache.commons.compress.utils.SeekableInMemoryByteChannel}
+     * allows you to read from an in-memory archive.</p>
+     *
+     * @param channel the channel to read
+     * @param password optional password if the archive is encrypted
+     * @param options the options to apply
+     * @throws IOException if reading the archive fails or the memory limit (if set) is too small
+     * @since 1.19
+     */
+    public SevenZFile(final SeekableByteChannel channel, final char[] password, final SevenZFileOptions options)
+            throws IOException {
+        this(channel, DEFAULT_FILE_NAME, password, options);
+    }
+
+    /**
+     * Reads a SeekableByteChannel as 7z archive
+     *
+     * <p>{@link
+     * org.apache.commons.compress.utils.SeekableInMemoryByteChannel}
+     * allows you to read from an in-memory archive.</p>
+     *
+     * @param channel the channel to read
+     * @param fileName name of the archive - only used for error reporting
+     * @param password optional password if the archive is encrypted
+     * @throws IOException if reading the archive fails
+     * @since 1.17
+     */
+    public SevenZFile(final SeekableByteChannel channel, String fileName,
+                      final char[] password) throws IOException {
+        this(channel, fileName, password, SevenZFileOptions.DEFAULT);
+    }
+
+    /**
+     * Reads a SeekableByteChannel as 7z archive with addtional options.
+     *
+     * <p>{@link
+     * org.apache.commons.compress.utils.SeekableInMemoryByteChannel}
+     * allows you to read from an in-memory archive.</p>
+     *
+     * @param channel the channel to read
+     * @param fileName name of the archive - only used for error reporting
+     * @param password optional password if the archive is encrypted
+     * @param options the options to apply
+     * @throws IOException if reading the archive fails or the memory limit (if set) is too small
+     * @since 1.19
+     */
+    public SevenZFile(final SeekableByteChannel channel, String fileName, final char[] password,
+            final SevenZFileOptions options) throws IOException {
+        this(channel, fileName, utf16Decode(password), false, options);
+    }
+
+    /**
+     * Reads a SeekableByteChannel as 7z archive
+     *
+     * <p>{@link
+     * org.apache.commons.compress.utils.SeekableInMemoryByteChannel}
+     * allows you to read from an in-memory archive.</p>
+     *
+     * @param channel the channel to read
+     * @param fileName name of the archive - only used for error reporting
+     * @throws IOException if reading the archive fails
+     * @since 1.17
+     */
+    public SevenZFile(final SeekableByteChannel channel, String fileName)
+        throws IOException {
+        this(channel, fileName, SevenZFileOptions.DEFAULT);
+    }
+
+    /**
+     * Reads a SeekableByteChannel as 7z archive with additional options.
+     *
+     * <p>{@link
+     * org.apache.commons.compress.utils.SeekableInMemoryByteChannel}
+     * allows you to read from an in-memory archive.</p>
+     *
+     * @param channel the channel to read
+     * @param fileName name of the archive - only used for error reporting
+     * @param options the options to apply
+     * @throws IOException if reading the archive fails or the memory limit (if set) is too small
+     * @since 1.19
+     */
+    public SevenZFile(final SeekableByteChannel channel, String fileName, final SevenZFileOptions options)
+            throws IOException {
+        this(channel, fileName, null, false, options);
     }
 
     /**
@@ -130,10 +296,11 @@ public class SevenZFile implements Closeable {
      * representation of the password.
      * @throws IOException if reading the archive fails
      * @since 1.13
+     * @deprecated use the char[]-arg version for the password instead
      */
     public SevenZFile(final SeekableByteChannel channel,
                       final byte[] password) throws IOException {
-        this(channel, "unknown archive", password);
+        this(channel, DEFAULT_FILE_NAME, password);
     }
 
     /**
@@ -144,28 +311,29 @@ public class SevenZFile implements Closeable {
      * allows you to read from an in-memory archive.</p>
      *
      * @param channel the channel to read
-     * @param filename name of the archive - only used for error reporting
+     * @param fileName name of the archive - only used for error reporting
      * @param password optional password if the archive is encrypted -
      * the byte array is supposed to be the UTF16-LE encoded
      * representation of the password.
      * @throws IOException if reading the archive fails
      * @since 1.13
+     * @deprecated use the char[]-arg version for the password instead
      */
-    public SevenZFile(final SeekableByteChannel channel, String filename,
+    public SevenZFile(final SeekableByteChannel channel, String fileName,
                       final byte[] password) throws IOException {
-        this(channel, filename, password, false);
+        this(channel, fileName, password, false, SevenZFileOptions.DEFAULT);
     }
 
     private SevenZFile(final SeekableByteChannel channel, String filename,
-                       final byte[] password, boolean closeOnError) throws IOException {
+                       final byte[] password, boolean closeOnError, SevenZFileOptions options) throws IOException {
         boolean succeeded = false;
         this.channel = channel;
         this.fileName = filename;
+        this.options = options;
         try {
             archive = readHeaders(password);
             if (password != null) {
-                this.password = new byte[password.length];
-                System.arraycopy(password, 0, this.password, 0, password.length);
+                this.password = Arrays.copyOf(password, password.length);
             } else {
                 this.password = null;
             }
@@ -180,11 +348,23 @@ public class SevenZFile implements Closeable {
     /**
      * Reads a file as unencrypted 7z archive
      *
-     * @param filename the file to read
+     * @param fileName the file to read
      * @throws IOException if reading the archive fails
      */
-    public SevenZFile(final File filename) throws IOException {
-        this(filename, null);
+    public SevenZFile(final File fileName) throws IOException {
+        this(fileName, SevenZFileOptions.DEFAULT);
+    }
+
+    /**
+     * Reads a file as unencrypted 7z archive
+     *
+     * @param fileName the file to read
+     * @param options the options to apply
+     * @throws IOException if reading the archive fails or the memory limit (if set) is too small
+     * @since 1.19
+     */
+    public SevenZFile(final File fileName, final SevenZFileOptions options) throws IOException {
+        this(fileName, (char[]) null, options);
     }
 
     /**
@@ -219,7 +399,11 @@ public class SevenZFile implements Closeable {
         }
         ++currentEntryIndex;
         final SevenZArchiveEntry entry = archive.files[currentEntryIndex];
+        if (entry.getName() == null && options.getUseDefaultNameForUnnamedEntries()) {
+            entry.setName(getDefaultName());
+        }
         buildDecodingStream();
+        uncompressedBytesReadFromCurrentEntry = compressedBytesReadFromCurrentEntry = 0;
         return entry;
     }
 
@@ -259,11 +443,8 @@ public class SevenZFile implements Closeable {
 
         final long startHeaderCrc = 0xffffFFFFL & buf.getInt();
         final StartHeader startHeader = readStartHeader(startHeaderCrc);
-
+        assertFitsIntoInt("nextHeaderSize", startHeader.nextHeaderSize);
         final int nextHeaderSizeInt = (int) startHeader.nextHeaderSize;
-        if (nextHeaderSizeInt != startHeader.nextHeaderSize) {
-            throw new IOException("cannot handle nextHeaderSize " + startHeader.nextHeaderSize);
-        }
         channel.position(SIGNATURE_HEADER_SIZE + startHeader.nextHeaderOffset);
         buf = ByteBuffer.allocate(nextHeaderSizeInt).order(ByteOrder.LITTLE_ENDIAN);
         readFully(buf);
@@ -335,6 +516,7 @@ public class SevenZFile implements Closeable {
         int nid =  getUnsignedByte(input);
         while (nid != NID.kEnd) {
             final long propertySize = readUint64(input);
+            assertFitsIntoInt("propertySize", propertySize);
             final byte[] property = new byte[(int)propertySize];
             input.get(property);
             nid = getUnsignedByte(input);
@@ -359,12 +541,13 @@ public class SevenZFile implements Closeable {
                 throw new IOException("Multi input/output stream coders are not yet supported");
             }
             inputStreamStack = Coders.addDecoder(fileName, inputStreamStack, //NOSONAR
-                    folder.getUnpackSizeForCoder(coder), coder, password);
+                    folder.getUnpackSizeForCoder(coder), coder, password, options.getMaxMemoryLimitInKb());
         }
         if (folder.hasCrc) {
             inputStreamStack = new CRC32VerifyingInputStream(inputStreamStack,
                     folder.getUnpackSize(), folder.crc);
         }
+        assertFitsIntoInt("unpackSize", folder.getUnpackSize());
         final byte[] nextHeader = new byte[(int)folder.getUnpackSize()];
         try (DataInputStream nextHeaderInputStream = new DataInputStream(inputStreamStack)) {
             nextHeaderInputStream.readFully(nextHeader);
@@ -401,9 +584,11 @@ public class SevenZFile implements Closeable {
     private void readPackInfo(final ByteBuffer header, final Archive archive) throws IOException {
         archive.packPos = readUint64(header);
         final long numPackStreams = readUint64(header);
+        assertFitsIntoInt("numPackStreams", numPackStreams);
+        final int numPackStreamsInt = (int) numPackStreams;
         int nid = getUnsignedByte(header);
         if (nid == NID.kSize) {
-            archive.packSizes = new long[(int)numPackStreams];
+            archive.packSizes = new long[numPackStreamsInt];
             for (int i = 0; i < archive.packSizes.length; i++) {
                 archive.packSizes[i] = readUint64(header);
             }
@@ -411,9 +596,9 @@ public class SevenZFile implements Closeable {
         }
 
         if (nid == NID.kCRC) {
-            archive.packCrcsDefined = readAllOrBits(header, (int)numPackStreams);
-            archive.packCrcs = new long[(int)numPackStreams];
-            for (int i = 0; i < (int)numPackStreams; i++) {
+            archive.packCrcsDefined = readAllOrBits(header, numPackStreamsInt);
+            archive.packCrcs = new long[numPackStreamsInt];
+            for (int i = 0; i < numPackStreamsInt; i++) {
                 if (archive.packCrcsDefined.get(i)) {
                     archive.packCrcs[i] = 0xffffFFFFL & header.getInt();
                 }
@@ -433,13 +618,15 @@ public class SevenZFile implements Closeable {
             throw new IOException("Expected kFolder, got " + nid);
         }
         final long numFolders = readUint64(header);
-        final Folder[] folders = new Folder[(int)numFolders];
+        assertFitsIntoInt("numFolders", numFolders);
+        final int numFoldersInt = (int) numFolders;
+        final Folder[] folders = new Folder[numFoldersInt];
         archive.folders = folders;
         final int external = getUnsignedByte(header);
         if (external != 0) {
             throw new IOException("External unsupported");
         }
-        for (int i = 0; i < (int)numFolders; i++) {
+        for (int i = 0; i < numFoldersInt; i++) {
             folders[i] = readFolder(header);
         }
 
@@ -448,6 +635,7 @@ public class SevenZFile implements Closeable {
             throw new IOException("Expected kCodersUnpackSize, got " + nid);
         }
         for (final Folder folder : folders) {
+            assertFitsIntoInt("totalOutputStreams", folder.totalOutputStreams);
             folder.unpackSizes = new long[(int)folder.totalOutputStreams];
             for (int i = 0; i < folder.totalOutputStreams; i++) {
                 folder.unpackSizes[i] = readUint64(header);
@@ -456,8 +644,8 @@ public class SevenZFile implements Closeable {
 
         nid = getUnsignedByte(header);
         if (nid == NID.kCRC) {
-            final BitSet crcsDefined = readAllOrBits(header, (int)numFolders);
-            for (int i = 0; i < (int)numFolders; i++) {
+            final BitSet crcsDefined = readAllOrBits(header, numFoldersInt);
+            for (int i = 0; i < numFoldersInt; i++) {
                 if (crcsDefined.get(i)) {
                     folders[i].hasCrc = true;
                     folders[i].crc = 0xffffFFFFL & header.getInt();
@@ -485,6 +673,7 @@ public class SevenZFile implements Closeable {
             totalUnpackStreams = 0;
             for (final Folder folder : archive.folders) {
                 final long numStreams = readUint64(header);
+                assertFitsIntoInt("numStreams", numStreams);
                 folder.numUnpackSubStreams = (int)numStreams;
                 totalUnpackStreams += numStreams;
             }
@@ -561,6 +750,7 @@ public class SevenZFile implements Closeable {
         final Folder folder = new Folder();
 
         final long numCoders = readUint64(header);
+        assertFitsIntoInt("numCoders", numCoders);
         final Coder[] coders = new Coder[(int)numCoders];
         long totalInStreams = 0;
         long totalOutStreams = 0;
@@ -585,6 +775,7 @@ public class SevenZFile implements Closeable {
             totalOutStreams += coders[i].numOutStreams;
             if (hasAttributes) {
                 final long propertiesSize = readUint64(header);
+                assertFitsIntoInt("propertiesSize", propertiesSize);
                 coders[i].properties = new byte[(int)propertiesSize];
                 header.get(coders[i].properties);
             }
@@ -595,13 +786,16 @@ public class SevenZFile implements Closeable {
             }
         }
         folder.coders = coders;
+        assertFitsIntoInt("totalInStreams", totalInStreams);
         folder.totalInputStreams = totalInStreams;
+        assertFitsIntoInt("totalOutStreams", totalOutStreams);
         folder.totalOutputStreams = totalOutStreams;
 
         if (totalOutStreams == 0) {
             throw new IOException("Total output streams can't be 0");
         }
         final long numBindPairs = totalOutStreams - 1;
+        assertFitsIntoInt("numBindPairs", numBindPairs);
         final BindPair[] bindPairs = new BindPair[(int)numBindPairs];
         for (int i = 0; i < bindPairs.length; i++) {
             bindPairs[i] = new BindPair();
@@ -614,6 +808,7 @@ public class SevenZFile implements Closeable {
             throw new IOException("Total input streams can't be less than the number of bind pairs");
         }
         final long numPackedStreams = totalInStreams - numBindPairs;
+        assertFitsIntoInt("numPackedStreams", numPackedStreams);
         final long packedStreams[] = new long[(int)numPackedStreams];
         if (numPackedStreams == 1) {
             int i;
@@ -667,12 +862,13 @@ public class SevenZFile implements Closeable {
 
     private void readFilesInfo(final ByteBuffer header, final Archive archive) throws IOException {
         final long numFiles = readUint64(header);
+        assertFitsIntoInt("numFiles", numFiles);
         final SevenZArchiveEntry[] files = new SevenZArchiveEntry[(int)numFiles];
         for (int i = 0; i < files.length; i++) {
             files[i] = new SevenZArchiveEntry();
         }
         BitSet isEmptyStream = null;
-        BitSet isEmptyFile = null; 
+        BitSet isEmptyFile = null;
         BitSet isAnti = null;
         while (true) {
             final int propertyType = getUnsignedByte(header);
@@ -707,6 +903,7 @@ public class SevenZFile implements Closeable {
                     if (((size - 1) & 1) != 0) {
                         throw new IOException("File names length invalid");
                     }
+                    assertFitsIntoInt("file names length", size - 1);
                     final byte[] names = new byte[(int)(size - 1)];
                     header.get(names);
                     int nextFile = 0;
@@ -803,7 +1000,7 @@ public class SevenZFile implements Closeable {
         int nonEmptyFileCounter = 0;
         int emptyFileCounter = 0;
         for (int i = 0; i < files.length; i++) {
-            files[i].setHasStream(isEmptyStream == null ? true : !isEmptyStream.get(i));
+            files[i].setHasStream(isEmptyStream == null || !isEmptyStream.get(i));
             if (files[i].hasStream()) {
                 files[i].setDirectory(false);
                 files[i].setAntiItem(false);
@@ -812,8 +1009,8 @@ public class SevenZFile implements Closeable {
                 files[i].setSize(archive.subStreamsInfo.unpackSizes[nonEmptyFileCounter]);
                 ++nonEmptyFileCounter;
             } else {
-                files[i].setDirectory(isEmptyFile == null ? true : !isEmptyFile.get(emptyFileCounter));
-                files[i].setAntiItem(isAnti == null ? false : isAnti.get(emptyFileCounter));
+                files[i].setDirectory(isEmptyFile == null || !isEmptyFile.get(emptyFileCounter));
+                files[i].setAntiItem(isAnti != null && isAnti.get(emptyFileCounter));
                 files[i].setHasCrc(false);
                 files[i].setSize(0);
                 ++emptyFileCounter;
@@ -839,7 +1036,7 @@ public class SevenZFile implements Closeable {
         streamMap.packStreamOffsets = new long[numPackSizes];
         for (int i = 0; i < numPackSizes; i++) {
             streamMap.packStreamOffsets[i] = nextPackStreamOffset;
-            nextPackStreamOffset += archive.packSizes[i]; 
+            nextPackStreamOffset += archive.packSizes[i];
         }
 
         streamMap.folderFirstFileIndex = new int[numFolders];
@@ -920,10 +1117,33 @@ public class SevenZFile implements Closeable {
     private InputStream buildDecoderStack(final Folder folder, final long folderOffset,
                 final int firstPackStreamIndex, final SevenZArchiveEntry entry) throws IOException {
         channel.position(folderOffset);
-        InputStream inputStreamStack =
-            new BufferedInputStream(
+        InputStream inputStreamStack = new FilterInputStream(new BufferedInputStream(
               new BoundedSeekableByteChannelInputStream(channel,
-                  archive.packSizes[firstPackStreamIndex]));
+                  archive.packSizes[firstPackStreamIndex]))) {
+            @Override
+            public int read() throws IOException {
+                final int r = in.read();
+                if (r >= 0) {
+                    count(1);
+                }
+                return r;
+            }
+            @Override
+            public int read(final byte[] b) throws IOException {
+                return read(b, 0, b.length);
+            }
+            @Override
+            public int read(final byte[] b, final int off, final int len) throws IOException {
+                final int r = in.read(b, off, len);
+                if (r >= 0) {
+                    count(r);
+                }
+                return r;
+            }
+            private void count(int c) {
+                compressedBytesReadFromCurrentEntry += c;
+            }
+        };
         final LinkedList<SevenZMethodConfiguration> methods = new LinkedList<>();
         for (final Coder coder : folder.getOrderedCoders()) {
             if (coder.numInStreams != 1 || coder.numOutStreams != 1) {
@@ -931,7 +1151,7 @@ public class SevenZFile implements Closeable {
             }
             final SevenZMethod method = SevenZMethod.byId(coder.decompressionMethodId);
             inputStreamStack = Coders.addDecoder(fileName, inputStreamStack,
-                    folder.getUnpackSizeForCoder(coder), coder, password);
+                    folder.getUnpackSizeForCoder(coder), coder, password, options.getMaxMemoryLimitInKb());
             methods.addFirst(new SevenZMethodConfiguration(method,
                      Coders.findByMethod(method).getOptionsFromCoder(coder, inputStreamStack)));
         }
@@ -945,13 +1165,17 @@ public class SevenZFile implements Closeable {
 
     /**
      * Reads a byte of data.
-     * 
+     *
      * @return the byte read, or -1 if end of input is reached
      * @throws IOException
      *             if an I/O error has occurred
      */
     public int read() throws IOException {
-        return getCurrentStream().read();
+        int b = getCurrentStream().read();
+        if (b >= 0) {
+            uncompressedBytesReadFromCurrentEntry++;
+        }
+        return b;
     }
 
     private InputStream getCurrentStream() throws IOException {
@@ -969,6 +1193,7 @@ public class SevenZFile implements Closeable {
             try (final InputStream stream = deferredBlockStreams.remove(0)) {
                 IOUtils.skip(stream, Long.MAX_VALUE);
             }
+            compressedBytesReadFromCurrentEntry = 0;
         }
 
         return deferredBlockStreams.get(0);
@@ -976,7 +1201,7 @@ public class SevenZFile implements Closeable {
 
     /**
      * Reads data into an array of bytes.
-     * 
+     *
      * @param b the array to write data to
      * @return the number of bytes read, or -1 if end of input is reached
      * @throws IOException
@@ -988,7 +1213,7 @@ public class SevenZFile implements Closeable {
 
     /**
      * Reads data into an array of bytes.
-     * 
+     *
      * @param b the array to write data to
      * @param off offset into the buffer to start filling at
      * @param len of bytes to read
@@ -997,7 +1222,30 @@ public class SevenZFile implements Closeable {
      *             if an I/O error has occurred
      */
     public int read(final byte[] b, final int off, final int len) throws IOException {
-        return getCurrentStream().read(b, off, len);
+        int cnt = getCurrentStream().read(b, off, len);
+        if (cnt > 0) {
+            uncompressedBytesReadFromCurrentEntry += cnt;
+        }
+        return cnt;
+    }
+
+    /**
+     * Provides statistics for bytes read from the current entry.
+     *
+     * @return statistics for bytes read from the current entry
+     * @since 1.17
+     */
+    public InputStreamStatistics getStatisticsForCurrentEntry() {
+        return new InputStreamStatistics() {
+            @Override
+            public long getCompressedCount() {
+                return compressedBytesReadFromCurrentEntry;
+            }
+            @Override
+            public long getUncompressedCount() {
+                return uncompressedBytesReadFromCurrentEntry;
+            }
+        };
     }
 
     private static long readUint64(final ByteBuffer in) throws IOException {
@@ -1066,4 +1314,55 @@ public class SevenZFile implements Closeable {
     public String toString() {
       return archive.toString();
     }
+
+    /**
+     * Derives a default file name from the archive name - if known.
+     *
+     * <p>This implements the same heuristics the 7z tools use. In
+     * 7z's case if an archive contains entries without a name -
+     * i.e. {@link SevenZArchiveEntry#getName} returns {@code null} -
+     * then its command line and GUI tools will use this default name
+     * when extracting the entries.</p>
+     *
+     * @return null if the name of the archive is unknown. Otherwise
+     * if the name of the archive has got any extension, it is
+     * stripped and the remainder returned. Finally if the name of the
+     * archive hasn't got any extension then a {@code ~} character is
+     * appended to the archive name.
+     *
+     * @since 1.19
+     */
+    public String getDefaultName() {
+        if (DEFAULT_FILE_NAME.equals(fileName) || fileName == null) {
+            return null;
+        }
+
+        final String lastSegment = new File(fileName).getName();
+        final int dotPos = lastSegment.lastIndexOf(".");
+        if (dotPos > 0) { // if the file starts with a dot then this is not an extension
+            return lastSegment.substring(0, dotPos);
+        }
+        return lastSegment + "~";
+    }
+
+    private static final CharsetEncoder PASSWORD_ENCODER = StandardCharsets.UTF_16LE.newEncoder();
+
+    private static byte[] utf16Decode(char[] chars) throws IOException {
+        if (chars == null) {
+            return null;
+        }
+        ByteBuffer encoded = PASSWORD_ENCODER.encode(CharBuffer.wrap(chars));
+        if (encoded.hasArray()) {
+            return encoded.array();
+        }
+        byte[] e = new byte[encoded.remaining()];
+        encoded.get(e);
+        return e;
+    }
+
+    private static void assertFitsIntoInt(String what, long value) throws IOException {
+        if (value > Integer.MAX_VALUE) {
+            throw new IOException("Cannot handle " + what + value);
+        }
+    }
 }
