@@ -80,7 +80,7 @@ import static org.apache.commons.compress.archivers.zip.ZipConstants.ZIP64_MAGIC
  */
 public class ZipArchiveInputStream extends ArchiveInputStream implements InputStreamStatistics {
 
-    /** The zip encoding to use for filenames and the file comment. */
+    /** The zip encoding to use for file names and the file comment. */
     private final ZipEncoding zipEncoding;
 
     // the provided encoding (for unit tests)
@@ -246,7 +246,7 @@ public class ZipArchiveInputStream extends ArchiveInputStream implements InputSt
             } else {
                 readFully(lfhBuf);
             }
-        } catch (final EOFException e) {
+        } catch (final EOFException e) { //NOSONAR
             return null;
         }
 
@@ -393,10 +393,10 @@ public class ZipArchiveInputStream extends ArchiveInputStream implements InputSt
         current.usesZip64 = z64 != null;
         if (!current.hasDataDescriptor) {
             if (z64 != null // same as current.usesZip64 but avoids NPE warning
-                    && (cSize.equals(ZipLong.ZIP64_MAGIC) || size.equals(ZipLong.ZIP64_MAGIC)) ) {
+                    && (ZipLong.ZIP64_MAGIC.equals(cSize) || ZipLong.ZIP64_MAGIC.equals(size)) ) {
                 current.entry.setCompressedSize(z64.getCompressedSize().getLongValue());
                 current.entry.setSize(z64.getSize().getLongValue());
-            } else {
+            } else if (cSize != null && size != null) {
                 current.entry.setCompressedSize(cSize.getValue());
                 current.entry.setSize(size.getValue());
             }
@@ -865,6 +865,12 @@ public class ZipArchiveInputStream extends ArchiveInputStream implements InputSt
                 && entry.getMethod() == ZipEntry.STORED);
     }
 
+    private static final String USE_ZIPFILE_INSTEAD_OF_STREAM_DISCLAIMER =
+        " while reading a stored entry using data descriptor. Either the archive is broken"
+        + " or it can not be read using ZipArchiveInputStream and you must use ZipFile."
+        + " A common cause for this is a ZIP archive containing a ZIP archive."
+        + " See http://commons.apache.org/proper/commons-compress/zip.html#ZipArchiveInputStream_vs_ZipFile";
+
     /**
      * Caches a stored entry that uses the data descriptor.
      *
@@ -908,8 +914,15 @@ public class ZipArchiveInputStream extends ArchiveInputStream implements InputSt
                 off = cacheBytesRead(bos, off, r, ddLen);
             }
         }
-
+        if (current.entry.getCompressedSize() != current.entry.getSize()) {
+            throw new ZipException("compressed and uncompressed size don't match"
+                                   + USE_ZIPFILE_INSTEAD_OF_STREAM_DISCLAIMER);
+        }
         final byte[] b = bos.toByteArray();
+        if (b.length != current.entry.getSize()) {
+            throw new ZipException("actual and claimed size don't match"
+                                   + USE_ZIPFILE_INSTEAD_OF_STREAM_DISCLAIMER);
+        }
         lastStoredEntry = new ByteArrayInputStream(b);
     }
 
@@ -929,18 +942,18 @@ public class ZipArchiveInputStream extends ArchiveInputStream implements InputSt
             throws IOException {
 
         boolean done = false;
-        int readTooMuch = 0;
         for (int i = 0; !done && i < offset + lastRead - 4; i++) {
             if (buf.array()[i] == LFH[0] && buf.array()[i + 1] == LFH[1]) {
-                if ((buf.array()[i + 2] == LFH[2] && buf.array()[i + 3] == LFH[3])
+                int expectDDPos = i;
+                if (i >= expectedDDLen &&
+                    (buf.array()[i + 2] == LFH[2] && buf.array()[i + 3] == LFH[3])
                     || (buf.array()[i] == CFH[2] && buf.array()[i + 3] == CFH[3])) {
                     // found a LFH or CFH:
-                    readTooMuch = offset + lastRead - i - expectedDDLen;
+                    expectDDPos = i - expectedDDLen;
                     done = true;
                 }
                 else if (buf.array()[i + 2] == DD[2] && buf.array()[i + 3] == DD[3]) {
                     // found DD:
-                    readTooMuch = offset + lastRead - i;
                     done = true;
                 }
                 if (done) {
@@ -948,8 +961,8 @@ public class ZipArchiveInputStream extends ArchiveInputStream implements InputSt
                     //   descriptor
                     // * copy the remaining bytes to cache
                     // * read data descriptor
-                    pushback(buf.array(), offset + lastRead - readTooMuch, readTooMuch);
-                    bos.write(buf.array(), 0, i);
+                    pushback(buf.array(), expectDDPos, offset + lastRead - expectDDPos);
+                    bos.write(buf.array(), 0, expectDDPos);
                     readDataDescriptor();
                 }
             }
@@ -1120,7 +1133,7 @@ public class ZipArchiveInputStream extends ArchiveInputStream implements InputSt
         // LFH has already been read and all but the first eight bytes contain (part of) the APK signing block,
         // also subtract 16 bytes in order to position us at the magic string
         BigInteger toSkip = len.add(BigInteger.valueOf(DWORD - suspectLocalFileHeader.length
-            - APK_SIGNING_BLOCK_MAGIC.length));
+            - (long) APK_SIGNING_BLOCK_MAGIC.length));
         byte[] magic = new byte[APK_SIGNING_BLOCK_MAGIC.length];
 
         try {
@@ -1144,7 +1157,7 @@ public class ZipArchiveInputStream extends ArchiveInputStream implements InputSt
                 realSkip(toSkip.longValue());
                 readFully(magic);
             }
-        } catch (EOFException ex) {
+        } catch (EOFException ex) { //NOSONAR
             // length was invalid
             return false;
         }
