@@ -25,51 +25,39 @@ import java.util.Arrays;
 
 import org.apache.commons.compress.compressors.CompressorInputStream;
 import org.apache.commons.compress.utils.BoundedInputStream;
-import org.apache.commons.compress.utils.ByteUtils;
-import org.apache.commons.compress.utils.CountingInputStream;
 import org.apache.commons.compress.utils.IOUtils;
-import org.apache.commons.compress.utils.InputStreamStatistics;
 
 /**
  * CompressorInputStream for the framing Snappy format.
  *
  * <p>Based on the "spec" in the version "Last revised: 2013-10-25"</p>
  *
- * @see <a href="https://github.com/google/snappy/blob/master/framing_format.txt">Snappy framing format description</a>
+ * @see <a href="http://code.google.com/p/snappy/source/browse/trunk/framing_format.txt">Snappy framing format description</a>
  * @since 1.7
  */
-public class FramedSnappyCompressorInputStream extends CompressorInputStream
-    implements InputStreamStatistics {
-
+public class FramedSnappyCompressorInputStream extends CompressorInputStream {
     /**
      * package private for tests only.
      */
     static final long MASK_OFFSET = 0xa282ead8L;
 
     private static final int STREAM_IDENTIFIER_TYPE = 0xff;
-    static final int COMPRESSED_CHUNK_TYPE = 0;
+    private static final int COMPRESSED_CHUNK_TYPE = 0;
     private static final int UNCOMPRESSED_CHUNK_TYPE = 1;
     private static final int PADDING_CHUNK_TYPE = 0xfe;
     private static final int MIN_UNSKIPPABLE_TYPE = 2;
     private static final int MAX_UNSKIPPABLE_TYPE = 0x7f;
     private static final int MAX_SKIPPABLE_TYPE = 0xfd;
 
-    // used by FramedSnappyCompressorOutputStream as well
-    static final byte[] SZ_SIGNATURE = new byte[] { //NOSONAR
+    private static final byte[] SZ_SIGNATURE = new byte[] {
         (byte) STREAM_IDENTIFIER_TYPE, // tag
         6, 0, 0, // length
         's', 'N', 'a', 'P', 'p', 'Y'
     };
 
-    private long unreadBytes;
-    private final CountingInputStream countingStream;
-
     /** The underlying stream to read compressed data from */
     private final PushbackInputStream in;
 
-    /** The dialect to expect */
-    private final FramedSnappyDialect dialect;
-
     private SnappyCompressorInputStream currentCompressedChunk;
 
     // used in no-arg read method
@@ -79,60 +67,16 @@ public class FramedSnappyCompressorInputStream extends CompressorInputStream
 
     private int uncompressedBytesRemaining;
     private long expectedChecksum = -1;
-    private final int blockSize;
-    private final PureJavaCrc32C checksum = new PureJavaCrc32C();
-
-    private final ByteUtils.ByteSupplier supplier = new ByteUtils.ByteSupplier() {
-        @Override
-        public int getAsByte() throws IOException {
-            return readOneByte();
-        }
-    };
-
-    /**
-     * Constructs a new input stream that decompresses
-     * snappy-framed-compressed data from the specified input stream
-     * using the {@link FramedSnappyDialect#STANDARD} dialect.
-     * @param in  the InputStream from which to read the compressed data
-     * @throws IOException if reading fails
-     */
-    public FramedSnappyCompressorInputStream(final InputStream in) throws IOException {
-        this(in, FramedSnappyDialect.STANDARD);
-    }
+    private PureJavaCrc32C checksum = new PureJavaCrc32C();
 
     /**
      * Constructs a new input stream that decompresses snappy-framed-compressed data
      * from the specified input stream.
      * @param in  the InputStream from which to read the compressed data
-     * @param dialect the dialect used by the compressed stream
-     * @throws IOException if reading fails
      */
-    public FramedSnappyCompressorInputStream(final InputStream in,
-                                             final FramedSnappyDialect dialect)
-        throws IOException {
-        this(in, SnappyCompressorInputStream.DEFAULT_BLOCK_SIZE, dialect);
-    }
-
-    /**
-     * Constructs a new input stream that decompresses snappy-framed-compressed data
-     * from the specified input stream.
-     * @param in  the InputStream from which to read the compressed data
-     * @param blockSize the block size to use for the compressed stream
-     * @param dialect the dialect used by the compressed stream
-     * @throws IOException if reading fails
-     * @since 1.14
-     */
-    public FramedSnappyCompressorInputStream(final InputStream in,
-                                             final int blockSize,
-                                             final FramedSnappyDialect dialect)
-        throws IOException {
-        countingStream = new CountingInputStream(in);
-        this.in = new PushbackInputStream(countingStream, 1);
-        this.blockSize = blockSize;
-        this.dialect = dialect;
-        if (dialect.hasStreamIdentifier()) {
-            readStreamIdentifier();
-        }
+    public FramedSnappyCompressorInputStream(InputStream in) throws IOException {
+        this.in = new PushbackInputStream(in, 1);
+        readStreamIdentifier();
     }
 
     /** {@inheritDoc} */
@@ -144,19 +88,16 @@ public class FramedSnappyCompressorInputStream extends CompressorInputStream
     /** {@inheritDoc} */
     @Override
     public void close() throws IOException {
-        try {
-            if (currentCompressedChunk != null) {
-                currentCompressedChunk.close();
-                currentCompressedChunk = null;
-            }
-        } finally {
-            in.close();
+        if (currentCompressedChunk != null) {
+            currentCompressedChunk.close();
+            currentCompressedChunk = null;
         }
+        in.close();
     }
 
     /** {@inheritDoc} */
     @Override
-    public int read(final byte[] b, final int off, final int len) throws IOException {
+    public int read(byte[] b, int off, int len) throws IOException {
         int read = readOnce(b, off, len);
         if (read == -1) {
             readNextBlock();
@@ -180,14 +121,6 @@ public class FramedSnappyCompressorInputStream extends CompressorInputStream
         return 0;
     }
 
-    /**
-     * @since 1.17
-     */
-    @Override
-    public long getCompressedCount() {
-        return countingStream.getBytesRead() - unreadBytes;
-    }
-
     /**
      * Read from the current chunk into the given array.
      *
@@ -195,10 +128,10 @@ public class FramedSnappyCompressorInputStream extends CompressorInputStream
      * read from the current chunk (which may be -1 if the end of the
      * chunk is reached).
      */
-    private int readOnce(final byte[] b, final int off, final int len) throws IOException {
+    private int readOnce(byte[] b, int off, int len) throws IOException {
         int read = -1;
         if (inUncompressedChunk) {
-            final int amount = Math.min(uncompressedBytesRemaining, len);
+            int amount = Math.min(uncompressedBytesRemaining, len);
             if (amount == 0) {
                 return -1;
             }
@@ -208,7 +141,7 @@ public class FramedSnappyCompressorInputStream extends CompressorInputStream
                 count(read);
             }
         } else if (currentCompressedChunk != null) {
-            final long before = currentCompressedChunk.getBytesRead();
+            long before = currentCompressedChunk.getBytesRead();
             read = currentCompressedChunk.read(b, off, len);
             if (read == -1) {
                 currentCompressedChunk.close();
@@ -226,12 +159,11 @@ public class FramedSnappyCompressorInputStream extends CompressorInputStream
     private void readNextBlock() throws IOException {
         verifyLastChecksumAndReset();
         inUncompressedChunk = false;
-        final int type = readOneByte();
+        int type = readOneByte();
         if (type == -1) {
             endReached = true;
         } else if (type == STREAM_IDENTIFIER_TYPE) {
             in.unread(type);
-            unreadBytes++;
             pushedBackBytes(1);
             readStreamIdentifier();
             readNextBlock();
@@ -248,15 +180,10 @@ public class FramedSnappyCompressorInputStream extends CompressorInputStream
             uncompressedBytesRemaining = readSize() - 4 /* CRC */;
             expectedChecksum = unmask(readCrc());
         } else if (type == COMPRESSED_CHUNK_TYPE) {
-            final boolean expectChecksum = dialect.usesChecksumWithCompressedChunks();
-            final long size = readSize() - (expectChecksum ? 4L : 0L);
-            if (expectChecksum) {
-                expectedChecksum = unmask(readCrc());
-            } else {
-                expectedChecksum = -1;
-            }
+            long size = readSize() - 4 /* CRC */;
+            expectedChecksum = unmask(readCrc());
             currentCompressedChunk =
-                new SnappyCompressorInputStream(new BoundedInputStream(in, size), blockSize);
+                new SnappyCompressorInputStream(new BoundedInputStream(in, size));
             // constructor reads uncompressed size
             count(currentCompressedChunk.getBytesRead());
         } else {
@@ -267,13 +194,17 @@ public class FramedSnappyCompressorInputStream extends CompressorInputStream
     }
 
     private long readCrc() throws IOException {
-        final byte[] b = new byte[4];
-        final int read = IOUtils.readFully(in, b);
+        byte[] b = new byte[4];
+        int read = IOUtils.readFully(in, b);
         count(read);
         if (read != 4) {
             throw new IOException("premature end of stream");
         }
-        return ByteUtils.fromLittleEndian(b);
+        long crc = 0;
+        for (int i = 0; i < 4; i++) {
+            crc |= (b[i] & 0xFFL) << (8 * i);
+        }
+        return crc;
     }
 
     static long unmask(long x) {
@@ -285,12 +216,21 @@ public class FramedSnappyCompressorInputStream extends CompressorInputStream
     }
 
     private int readSize() throws IOException {
-        return (int) ByteUtils.fromLittleEndian(supplier, 3);
+        int b = 0;
+        int sz = 0;
+        for (int i = 0; i < 3; i++) {
+            b = readOneByte();
+            if (b == -1) {
+                throw new IOException("premature end of stream");
+            }
+            sz |= (b << (i * 8));
+        }
+        return sz;
     }
 
     private void skipBlock() throws IOException {
-        final int size = readSize();
-        final long read = IOUtils.skip(in, size);
+        int size = readSize();
+        long read = IOUtils.skip(in, size);
         count(read);
         if (read != size) {
             throw new IOException("premature end of stream");
@@ -298,8 +238,8 @@ public class FramedSnappyCompressorInputStream extends CompressorInputStream
     }
 
     private void readStreamIdentifier() throws IOException {
-        final byte[] b = new byte[10];
-        final int read = IOUtils.readFully(in, b);
+        byte[] b = new byte[10];
+        int read = IOUtils.readFully(in, b);
         count(read);
         if (10 != read || !matches(b, 10)) {
             throw new IOException("Not a framed Snappy stream");
@@ -307,7 +247,7 @@ public class FramedSnappyCompressorInputStream extends CompressorInputStream
     }
 
     private int readOneByte() throws IOException {
-        final int b = in.read();
+        int b = in.read();
         if (b != -1) {
             count(1);
             return b & 0xFF;
@@ -327,12 +267,12 @@ public class FramedSnappyCompressorInputStream extends CompressorInputStream
      * Checks if the signature matches what is expected for a .sz file.
      *
      * <p>.sz files start with a chunk with tag 0xff and content sNaPpY.</p>
-     *
+     * 
      * @param signature the bytes to check
      * @param length    the number of bytes to check
      * @return          true if this is a .sz stream, false otherwise
      */
-    public static boolean matches(final byte[] signature, final int length) {
+    public static boolean matches(byte[] signature, int length) {
 
         if (length < SZ_SIGNATURE.length) {
             return false;
