@@ -18,8 +18,6 @@
  */
 package org.apache.commons.compress.archivers;
 
-import static org.junit.Assert.*;
-
 import java.io.ByteArrayOutputStream;
 import java.io.File;
 import java.io.FileInputStream;
@@ -34,20 +32,28 @@ import org.apache.commons.compress.archivers.jar.JarArchiveEntry;
 import org.apache.commons.compress.archivers.tar.TarArchiveEntry;
 import org.apache.commons.compress.archivers.zip.ZipArchiveEntry;
 import org.apache.commons.compress.utils.IOUtils;
-import org.junit.Test;
 
 public class ArchiveOutputStreamTest extends AbstractTestCase {
 
-    @Test
+    @Override
+    protected void setUp() throws Exception {
+        super.setUp();
+    }
+
+    @Override
+    protected void tearDown() throws Exception {
+        super.tearDown();
+    }
+
     public void testFinish() throws Exception {
-        final OutputStream out1 = new ByteArrayOutputStream();
+        OutputStream out1 = new ByteArrayOutputStream();
 
         ArchiveOutputStream aos1 = factory.createArchiveOutputStream("zip", out1);
         aos1.putArchiveEntry(new ZipArchiveEntry("dummy"));
         try {
             aos1.finish();
             fail("After putArchive should follow closeArchive");
-        } catch (final IOException io) {
+        } catch (IOException io) {
             // Exception expected
         }
 
@@ -56,7 +62,7 @@ public class ArchiveOutputStreamTest extends AbstractTestCase {
         try {
             aos1.finish();
             fail("After putArchive should follow closeArchive");
-        } catch (final IOException io) {
+        } catch (IOException io) {
             // Exception expected
         }
 
@@ -65,7 +71,7 @@ public class ArchiveOutputStreamTest extends AbstractTestCase {
         try {
             aos1.finish();
             fail("After putArchive should follow closeArchive");
-        } catch (final IOException io) {
+        } catch (IOException io) {
             // Exception expected
         }
 
@@ -74,7 +80,7 @@ public class ArchiveOutputStreamTest extends AbstractTestCase {
         try {
             aos1.finish();
             fail("After putArchive should follow closeArchive");
-        } catch (final IOException io) {
+        } catch (IOException io) {
             // Exception expected
         }
 
@@ -83,69 +89,61 @@ public class ArchiveOutputStreamTest extends AbstractTestCase {
         try {
             aos1.finish();
             fail("After putArchive should follow closeArchive");
-        } catch (final IOException io) {
+        } catch (IOException io) {
             // Exception expected
         }
     }
 
-    @Test
     public void testOptionalFinish() throws Exception {
-        final OutputStream out1 = new ByteArrayOutputStream();
+        OutputStream out1 = new ByteArrayOutputStream();
 
-        try (ArchiveOutputStream aos1 = factory.createArchiveOutputStream("zip", out1)) {
-            aos1.putArchiveEntry(new ZipArchiveEntry("dummy"));
-            aos1.closeArchiveEntry();
-        }
+        ArchiveOutputStream aos1 = factory.createArchiveOutputStream("zip", out1);
+        aos1.putArchiveEntry(new ZipArchiveEntry("dummy"));
+        aos1.closeArchiveEntry();
+        aos1.close();
 
-        final ArchiveOutputStream finishTest;
-        try (ArchiveOutputStream aos1 = factory.createArchiveOutputStream("jar", out1)) {
-            finishTest = aos1;
-            aos1.putArchiveEntry(new JarArchiveEntry("dummy"));
-            aos1.closeArchiveEntry();
-        }
+        aos1 = factory.createArchiveOutputStream("jar", out1);
+        aos1.putArchiveEntry(new JarArchiveEntry("dummy"));
+        aos1.closeArchiveEntry();
+        aos1.close();
         try {
-            finishTest.finish();
+            aos1.finish();
             fail("finish() cannot follow close()");
-        } catch (final IOException io) {
+        } catch (IOException io) {
             // Exception expected
         }
     }
 
-    @Test
     public void testCallSequenceAr() throws Exception{
         doCallSequence("Ar");
     }
 
-    @Test
     public void testCallSequenceCpio() throws Exception{
         doCallSequence("Cpio");
     }
 
-    @Test
     public void testCallSequenceJar() throws Exception{
         doCallSequence("Jar");
     }
 
-    @Test
     public void testCallSequenceTar() throws Exception{
         doCallSequence("Tar");
     }
 
-    @Test
     public void testCallSequenceZip() throws Exception{
         doCallSequence("Zip");
     }
 
-    private void doCallSequence(final String archiveType) throws Exception {
-        final OutputStream out1 = new ByteArrayOutputStream();
-        final File dummy = getFile("test1.xml"); // need a real file
+    private void doCallSequence(String archiveType) throws Exception {
+        OutputStream out1 = new ByteArrayOutputStream();
+        File dummy = getFile("test1.xml"); // need a real file
 
         ArchiveOutputStream aos1;
         aos1 = factory.createArchiveOutputStream(archiveType, out1);
         aos1.putArchiveEntry(aos1.createArchiveEntry(dummy, "dummy"));
-        try (InputStream is = new FileInputStream(dummy)) {
-            IOUtils.copy(is, aos1);
-        }
+        InputStream is = new FileInputStream(dummy);
+        IOUtils.copy(is, aos1);
+        is.close();
         aos1.closeArchiveEntry();
         aos1.close(); // omitted finish
 
@@ -155,38 +153,38 @@ public class ArchiveOutputStreamTest extends AbstractTestCase {
         try {
             aos1.closeArchiveEntry();
             fail("Should have raised IOException - closeArchiveEntry() called before putArchiveEntry()");
-        } catch (final IOException expected) {
+        } catch (IOException expected) {
         }
 
         aos1.putArchiveEntry(aos1.createArchiveEntry(dummy, "dummy"));
-        try (InputStream is = new FileInputStream(dummy)) {
-            IOUtils.copy(is, aos1);
-        }
+        is = new FileInputStream(dummy);
+        IOUtils.copy(is, aos1);
+        is.close();
 
         // TODO check if second putArchiveEntry() can follow without closeAE?
 
         try {
             aos1.finish();
             fail("Should have raised IOException - finish() called before closeArchiveEntry()");
-        } catch (final IOException expected) {
+        } catch (IOException expected) {
         }
         try {
             aos1.close();
             fail("Should have raised IOException - close() called before closeArchiveEntry()");
-        } catch (final IOException expected) {
+        } catch (IOException expected) {
         }
         aos1.closeArchiveEntry();
         try {
             aos1.closeArchiveEntry();
             fail("Should have raised IOException - closeArchiveEntry() called with no open entry");
-        } catch (final IOException expected) {
+        } catch (IOException expected) {
         }
         aos1.finish();
         aos1.close();
         try {
             aos1.finish();
             fail("Should have raised IOException - finish() called after close()");
-        } catch (final IOException expected) {
+        } catch (IOException expected) {
         }
     }
 }
